import sys

from src.roadmapper.roadmap import Roadmap
from src.roadmapper.timelinemode import TimelineMode
from src.tests.roadmap_generators.roadmap_abc import RoadmapABC


class MilestoneTextAlignment(RoadmapABC):

    def generate_and_save_as(
            self,
            file_name: str = "example.png",
    ) -> None:
        roadmap: Roadmap = self.generate()
        roadmap.draw()
        roadmap.save(file_name)

    def generate(self) -> Roadmap:
        width = 1200
        height = 1000
        auto_height = True
        mode = TimelineMode.MONTHLY
        start_date = "2023-01-01"
        number_of_items = 12
        show_generic_dates = False

        roadmap = Roadmap(width, height, auto_height=auto_height, show_marker=False)

        roadmap.set_title("2023 Milestone Alignment")
        roadmap.set_subtitle("GodZone Corporation")

        roadmap.set_timeline(
            mode=mode,
            start=start_date,
            number_of_items=number_of_items,
            show_generic_dates=show_generic_dates,
        )

        group = roadmap.add_group("Milestone Alignment", text_alignment="left")
        # Group containing each version of left align milsetone text
        left_task = group.add_task(
            "Left align", start_date, "2023-12-31"
        )
        left_task.add_milestone(
            "default left", "2023-05-01", text_alignment="left"
        )
        left_task.add_milestone(
            "percent left", "2023-07-01", text_alignment="left:75%"
        )
        left_task.add_milestone(
            "units left", "2023-11-01", text_alignment="left:10"
        )

        # Group containing each version of right align milestone text
        right_task = group.add_task(
            "Right align", start_date, "2023-12-31"
        )
        right_task.add_milestone(
            "default right", "2023-02-01", text_alignment="right"
        )
        right_task.add_milestone(
            "percent right", "2023-05-01", text_alignment="right:75%"
        )
        right_task.add_milestone(
            "units right", "2023-08-01", text_alignment="right:10"
        )

        # Group containing each version of none align milestone text
        centre_task = group.add_task(
            "Centre align", start_date, "2023-12-31"
        )
        centre_task.add_milestone(
            "not specified (centre)", "2023-02-15"
        )
        centre_task.add_milestone(
            "default centre", "2023-08-01", text_alignment="centre"
        )


        roadmap.set_footer("Generated by Roadmapper")

        return roadmap


def is_arg_given():
    return len(sys.argv) > 1


if __name__ == '__main__':
    if is_arg_given():
        example_name = sys.argv[1]
        MilestoneTextAlignment().generate_and_save_as(file_name=example_name)
    else:
        MilestoneTextAlignment.generate_and_save_as()
